//
//  LBAssetCell.swift
//  LBAssetPickerDemo
//
//  Created by zhubaotown on 16/4/22.
//  Copyright © 2016年 zhubaotown. All rights reserved.
//

import UIKit
import AssetsLibrary

class LBAssetCell: UICollectionViewCell {

    private var iconView : UIImageView? = nil;
    private var chooseIcon : UIImageView? = nil;
    private var videoTitle : LBAssetVideoTitle? = nil;
    private var bgView : UIView? = nil;
    
    private var assetData : ALAsset? = nil;
    
    private var _isChoosed : Bool = false;
    var isChoosed : Bool {
        get {
            return _isChoosed;
        }
        set(newValue) {
            if _isDisabled {
                return;
            }
            _isChoosed = newValue;
            self.chooseIcon?.hidden = !_isChoosed;
            if _isDisabled {
                self.chooseIcon?.hidden = true;
                self.bgView!.backgroundColor = UIColor(white:1.0, alpha:0.9);
            } else if self.isChoosed {
                self.bgView!.backgroundColor = UIColor(white: 1.0, alpha:0.3);
            } else {
                self.bgView!.backgroundColor = UIColor(white: 1.0, alpha:0.0);
            }
        }
    }
    
    private var _isDisabled : Bool = false;
    var isDisabled : Bool {
        get {
            return _isDisabled;
        }
        set(newValue) {
            _isDisabled = newValue;
            if _isDisabled {
                self.chooseIcon?.hidden = true;
                self.bgView!.backgroundColor = UIColor(white:1.0, alpha:0.9);
            } else if self.isChoosed {
                self.bgView!.backgroundColor = UIColor(white: 1.0, alpha:0.3);
            } else {
                self.bgView!.backgroundColor = UIColor(white: 1.0, alpha:0.0);
            }
        }
    }
    
    override init(frame: CGRect) {
        super.init(frame: frame);
        
        self.iconView = UIImageView(frame: self.bounds);
        self.iconView!.backgroundColor = UIColor.clearColor();
        self.addSubview(self.iconView!);
        
        self.videoTitle = LBAssetVideoTitle(frame:CGRectMake(0,frame.size.height - 25,frame.size.width,25));
        self.videoTitle!.backgroundColor = UIColor.clearColor();
        self.videoTitle!.textColor = UIColor.whiteColor();
        self.videoTitle!.font = UIFont.systemFontOfSize(12.0);
        self.videoTitle!.textAlignment = .Right;
        self.videoTitle!.hidden = true;
        self.addSubview(self.videoTitle!);
        
        self.chooseIcon = UIImageView(frame:CGRectMake(0,0,26,26));
        self.chooseIcon?.center = CGPointMake(self.frame.size.width - 13.0,self.frame.size.height - 13.0);
        self.chooseIcon?.backgroundColor = UIColor.clearColor();
        self.chooseIcon?.image = UIImage(named: "LBAssetChooseIcon.png")
        self.addSubview(self.chooseIcon!);
        
        self.bgView = UIView(frame: self.bounds);
        self.bgView?.backgroundColor = UIColor.clearColor();
        self.addSubview(self.bgView!);
    }
    
    required init?(coder aDecoder: NSCoder) {
        fatalError("init(coder:) has not been implemented")
    }
    
    func bindData(asset : ALAsset) {
        self.assetData = asset;
        self.iconView?.image = UIImage(CGImage: asset.thumbnail().takeUnretainedValue());
        let aType : String = asset.valueForProperty(ALAssetPropertyType) as! String;
        if aType == ALAssetTypeVideo {
            self.videoTitle!.hidden = false;
            self.videoTitle!.text = NSDate.timeDescriptionOfTimeInterval(asset.valueForProperty(ALAssetPropertyDuration).doubleValue);
        }else{
            self.videoTitle!.hidden = true;
        }
    }

}

extension NSDate {
    class func componetsWithTimeInterval(timeInterval : NSTimeInterval) -> NSDateComponents {
        let calendar : NSCalendar = NSCalendar.currentCalendar();
        
        let date1 : NSDate = NSDate();
        let date2 : NSDate = NSDate(timeInterval: timeInterval, sinceDate:date1);
        let flags : NSCalendarUnit = [.Year,.Month,.Day,.Hour,.Minute];
        return calendar.components(flags, fromDate:date1, toDate:date2, options:NSCalendarOptions(rawValue:0));
    }
    
    class func timeDescriptionOfTimeInterval(timeInterval : NSTimeInterval) ->String? {
        let components : NSDateComponents = componetsWithTimeInterval(timeInterval);
        let roundedSeconds : NSInteger = lround(timeInterval - (Double(components.hour) * 60) - (Double(components.minute) * 60 * 60));
        
        if (components.hour > 0){
            return String(format:"%d:%02d:%02d",components.hour,components.minute,roundedSeconds);
        }else{
            return String(format:"%d:%02d",components.minute,roundedSeconds);
        }
    }
}
