//
//  LBAssetGroupViewController.swift
//  LBAssetPickerDemo
//
//  Created by zhubaotown on 16/4/21.
//  Copyright © 2016年 zhubaotown. All rights reserved.
//

import UIKit
import AssetsLibrary

class LBAssetGroupViewController: UITableViewController {
    
    private var assetsLibrary : ALAssetsLibrary? = nil;
    private var groupList : NSMutableArray? = nil;

    override func viewDidLoad() {
        super.viewDidLoad()
        
        self.edgesForExtendedLayout = .None;
        
        self.tableView.separatorStyle = .SingleLine;
        self.tableView.rowHeight = kThumbnailSize.height + 12;
        
        if (self.navigationController != nil
            && self.navigationController!.isKindOfClass(LBAssetPickerController.self)) {
            if (self.navigationController as! LBAssetPickerController).showCancelButton {
                let cancelBtn : UIBarButtonItem = UIBarButtonItem(title:"取消", style:.Plain, target:self, action:#selector(LBAssetGroupViewController.didDismissed));
                self.navigationItem.rightBarButtonItem = cancelBtn;
            }
        }
        
        setNavTitle("相册");
        
        initData();
    }

    override func didReceiveMemoryWarning() {
        super.didReceiveMemoryWarning()
        // Dispose of any resources that can be recreated.
    }
    
    override func shouldAutorotate() -> Bool {
        return true;
    }
    
    override func supportedInterfaceOrientations() -> UIInterfaceOrientationMask {
        return .AllButUpsideDown;
    }
    
    //MARK:- 初始化数据
    private func initData() {
        if self.assetsLibrary == nil {
            self.assetsLibrary = LBAssetGroupViewController.defaultAssetsLibrary();
        }
        if self.groupList == nil {
            self.groupList = NSMutableArray(capacity: 2);
        }else{
            self.groupList!.removeAllObjects();
        }
        
        weak var weakSelf : LBAssetGroupViewController? = self;
        let assetsPicker : LBAssetPickerController = self.navigationController as! LBAssetPickerController;
        let assetsFilter : ALAssetsFilter! = assetsPicker.assetsFilter;
        let resultBlock : ALAssetsLibraryGroupsEnumerationResultsBlock = { (group : ALAssetsGroup!, stop : UnsafeMutablePointer<ObjCBool>) in
            if group != nil {
                group.setAssetsFilter(assetsFilter);
                if (group.numberOfAssets() > 0 || assetsPicker.showEmptyGroups) {
                    weakSelf?.groupList!.addObject(group);
                }else{
                    weakSelf?.reloadData();
                }
            }else{
                weakSelf?.reloadData();
            }
        };
        let failureBlock : ALAssetsLibraryAccessFailureBlock = { (err : NSError!) in
            weakSelf?.showNotAllowdView();
        };
        
        self.assetsLibrary!.enumerateGroupsWithTypes(ALAssetsGroupType(ALAssetsGroupSavedPhotos),usingBlock:resultBlock,failureBlock:failureBlock);
        
        let type : UInt32 = ALAssetsGroupLibrary | ALAssetsGroupAlbum | ALAssetsGroupEvent | ALAssetsGroupFaces | ALAssetsGroupPhotoStream;
        self.assetsLibrary!.enumerateGroupsWithTypes(ALAssetsGroupType(type), usingBlock:resultBlock, failureBlock:failureBlock);
    }
    
    //MARK:- Method
    private func setNavTitle(aTitle : String?) {
        let aLab : UILabel = UILabel(frame:CGRectMake(0, 0, 100, 40));
        aLab.textAlignment = .Center;
        aLab.font = UIFont.boldSystemFontOfSize(18.0);
        aLab.lineBreakMode = .ByTruncatingTail;
        aLab.textColor = UIColor.blackColor();
        aLab.backgroundColor = UIColor.clearColor();
        aLab.text = aTitle;
        self.navigationItem.titleView = aLab;
    }
    
    private class func defaultAssetsLibrary() -> ALAssetsLibrary {
        struct GroupStatic {
            static var library : ALAssetsLibrary? = nil;
            static var onceToken : dispatch_once_t = 0;
        }
        
        dispatch_once(&GroupStatic.onceToken) {
            GroupStatic.library = ALAssetsLibrary();
        }
        
        return GroupStatic.library!;
    }
    
    func didDismissed() {
        let picker : LBAssetPickerController = self.navigationController as! LBAssetPickerController;
        picker.pickerDelegate?.assetPickerControllerDidCanceled?(picker);
        picker.dismissViewControllerAnimated(true, completion:nil);
    }
    
    //Data show Method
    func showNotAllowdView() {
        self.title = nil;
        let lockView : UIImageView = UIImageView(image:UIImage(named:"LBAssetsLockedIcon.png"));
        lockView.translatesAutoresizingMaskIntoConstraints = false;
        
        let titleLab : UILabel = UILabel();
        titleLab.translatesAutoresizingMaskIntoConstraints = false;
        titleLab.preferredMaxLayoutWidth = 300.0;
        titleLab.text = "此应用无法使用您的照片或视频。";
        titleLab.font = UIFont.boldSystemFontOfSize(17.0);
        titleLab.textColor = UIColor(red:129.0/255.0, green:136.0/255.0, blue:148.0/255.0, alpha:1.0);
        titleLab.numberOfLines = 0;
        titleLab.textAlignment = .Center;
        
        let msgLab : UILabel = UILabel();
        msgLab.translatesAutoresizingMaskIntoConstraints = false;
        msgLab.preferredMaxLayoutWidth = 300.0;
        msgLab.text = "你可以在「隐私设置」中启用存取。";
        msgLab.font = UIFont.systemFontOfSize(14.0);
        msgLab.textColor = UIColor(red:129.0/255.0, green:136.0/255.0, blue:148.0/255.0, alpha:1.0);
        msgLab.numberOfLines = 0;
        msgLab.textAlignment = .Center;
        
        titleLab.sizeToFit();
        msgLab.sizeToFit();
        
        let centerView : UIView = UIView();
        centerView.translatesAutoresizingMaskIntoConstraints = false;
        centerView.addSubview(lockView);
        centerView.addSubview(titleLab);
        centerView.addSubview(msgLab);
        
        let viewsDict : NSDictionary = ["lockView" : lockView,"titleLab" : titleLab,"msgLab" : msgLab];
        
        centerView.addConstraint(NSLayoutConstraint(item:lockView, attribute:.CenterX, relatedBy:.Equal, toItem:centerView, attribute:.CenterX, multiplier:1.0, constant:0.0));
        centerView.addConstraint(NSLayoutConstraint(item:titleLab, attribute:.CenterX, relatedBy:.Equal, toItem:lockView, attribute:.CenterX, multiplier:1.0, constant:0.0));
        centerView.addConstraint(NSLayoutConstraint(item:msgLab, attribute:.CenterX, relatedBy:.Equal, toItem:lockView, attribute:.CenterX, multiplier:1.0, constant:0.0));
        centerView.addConstraints(NSLayoutConstraint.constraintsWithVisualFormat("V:|[lockView]-[titleLab]-[msgLab]|", options: NSLayoutFormatOptions(rawValue: 0), metrics:nil, views: (viewsDict as! [String : AnyObject])));
        
        let bgView : UIView = UIView();
        bgView.addSubview(centerView);
        bgView.addConstraint(NSLayoutConstraint(item:centerView, attribute:.CenterX, relatedBy:.Equal, toItem:bgView, attribute:.CenterX, multiplier:1.0, constant:0.0));
        bgView.addConstraint(NSLayoutConstraint(item:centerView, attribute:.CenterY, relatedBy:.Equal, toItem:bgView, attribute:.CenterY, multiplier:1.0, constant:0.0));
        
        self.tableView.backgroundView = bgView;
        
    }
    
    func showNotAssets() {
        let titleLab : UILabel = UILabel();
        titleLab.translatesAutoresizingMaskIntoConstraints = false;
        titleLab.preferredMaxLayoutWidth = 300.0;
        titleLab.text = "没有照片或视频。";
        titleLab.font = UIFont.boldSystemFontOfSize(26.0);
        titleLab.textColor = UIColor(red:153.0/255.0, green:153.0/255.0, blue:153.0/255.0, alpha:1.0);
        titleLab.numberOfLines = 0;
        titleLab.textAlignment = .Center;
        
        let msgLab : UILabel = UILabel();
        msgLab.translatesAutoresizingMaskIntoConstraints = false;
        msgLab.preferredMaxLayoutWidth = 300.0;
        msgLab.text = "您可以使用 iTunes 将照片和视频\n同步到 iPhone。";
        msgLab.font = UIFont.systemFontOfSize(18.0);
        msgLab.textColor = UIColor(red:153.0/255.0, green:153.0/255.0, blue:153.0/255.0, alpha:1.0);
        msgLab.numberOfLines = 0;
        msgLab.textAlignment = .Center;
        
        titleLab.sizeToFit();
        msgLab.sizeToFit();
        
        let centerView : UIView = UIView();
        centerView.translatesAutoresizingMaskIntoConstraints = false;
        centerView.addSubview(titleLab);
        centerView.addSubview(msgLab);
        
        let viewsDict : NSDictionary = ["titleLab" : titleLab,"msgLab" : msgLab];
        
        centerView.addConstraint(NSLayoutConstraint(item:titleLab, attribute:.CenterX, relatedBy:.Equal, toItem:centerView, attribute:.CenterX, multiplier:1.0, constant:0.0));
        centerView.addConstraint(NSLayoutConstraint(item:msgLab, attribute:.CenterX, relatedBy:.Equal, toItem:titleLab, attribute:.CenterX, multiplier:1.0, constant:0.0));
        centerView.addConstraints(NSLayoutConstraint.constraintsWithVisualFormat("V:|[titleLab]-[msgLab]|", options: NSLayoutFormatOptions(rawValue: 0), metrics:nil, views: (viewsDict as! [String : AnyObject])));
        
        let bgView : UIView = UIView();
        bgView.addSubview(centerView);
        bgView.addConstraint(NSLayoutConstraint(item:centerView, attribute:.CenterX, relatedBy:.Equal, toItem:bgView, attribute:.CenterX, multiplier:1.0, constant:0.0));
        bgView.addConstraint(NSLayoutConstraint(item:centerView, attribute:.CenterY, relatedBy:.Equal, toItem:bgView, attribute:.CenterY, multiplier:1.0, constant:0.0));
        
        self.tableView.backgroundView = bgView;
    }
    
    func reloadData() {
        if self.groupList?.count == 0 {
            showNotAssets();
        }else{
            self.tableView.backgroundView = nil;
        }
        self.tableView.reloadData();
    }
    
    //MARK:- TableView 
    override func numberOfSectionsInTableView(tableView: UITableView) -> Int {
        return 1;
    }
    
    override func tableView(tableView: UITableView, numberOfRowsInSection section: Int) -> Int {
        return (self.groupList?.count)!;
    }
    
    override func tableView(tableView: UITableView, heightForRowAtIndexPath indexPath: NSIndexPath) -> CGFloat {
        return kThumbnailSize.height + 12.0;
    }
    
    override func tableView(tableView: UITableView, cellForRowAtIndexPath indexPath: NSIndexPath) -> UITableViewCell {
        var cell : LBAssetGroupCell? = tableView.dequeueReusableCellWithIdentifier("groupCell") as? LBAssetGroupCell;
        if cell == nil {
            cell = LBAssetGroupCell(style:.Default, reuseIdentifier:"groupCell");
        }
        cell?.bindData(self.groupList?.objectAtIndex(indexPath.row) as! ALAssetsGroup);
        return cell!;
    }
    
    override func tableView(tableView: UITableView, didSelectRowAtIndexPath indexPath: NSIndexPath) {
        tableView.deselectRowAtIndexPath(indexPath, animated:true);
        
        //TODO::去相册里面
        let aCnt : LBAssetViewController = LBAssetViewController();
        aCnt.assetGroup = self.groupList?.objectAtIndex(indexPath.row) as? ALAssetsGroup;
        self.navigationController?.pushViewController(aCnt, animated:true);
    }

}
