//
//  LBAssetViewController.swift
//  LBAssetPickerDemo
//
//  Created by zhubaotown on 16/4/22.
//  Copyright © 2016年 zhubaotown. All rights reserved.
//

import UIKit
import AssetsLibrary

class LBAssetViewController:UIViewController,
    UICollectionViewDataSource,
UICollectionViewDelegateFlowLayout{
    
    var assetGroup : ALAssetsGroup? = nil;
    private var assetTitle : String? = nil;
    private var selectedItemList : NSMutableArray? = nil;
    private var assetList : NSMutableArray? = nil;
    
    private let itemColumsPerRow : NSInteger = 3;
    private var itemWidth : CGFloat = 0;
    private let miniItemSpacing : CGFloat = 5;
    private let miniLineSpacing : CGFloat = 5;
    private let leftSpacing : CGFloat = 15.0;
    private let screenSize : CGSize = UIScreen.mainScreen().bounds.size;
    
    private var photoNum : NSInteger = 0;
    private var videoNum : NSInteger = 0;
    private var choosePhotoNum : NSInteger = 0;
    private var chooseVideoNum : NSInteger = 0;
    
    private var isUnFirst : Bool = false;
    
    private var assetView : UICollectionView? = nil;
    private let cellId : String = "assetCell";
    private let headId : String = "assetHead";

    override func viewDidLoad() {
        super.viewDidLoad()
        
        let finishBtn : UIBarButtonItem = UIBarButtonItem(title:"完成", style:.Plain, target:self, action:#selector(LBAssetViewController.finishPickingAssets));
        self.navigationItem.rightBarButtonItem = finishBtn;
        
        self.title = "相册";
        self.view.backgroundColor = UIColor.whiteColor();
        
        self.itemWidth = (self.screenSize.width - self.leftSpacing * 2.0 - self.miniItemSpacing * (CGFloat(self.itemColumsPerRow) - 1)) / CGFloat(self.itemColumsPerRow);
        self.selectedItemList = NSMutableArray(capacity: 2);
        self.assetList = NSMutableArray(capacity: 2);
        
        let flowLayot : UICollectionViewFlowLayout = UICollectionViewFlowLayout();
        flowLayot.scrollDirection = .Vertical;
        self.assetView = UICollectionView(frame:CGRectMake(0,0,self.screenSize.width, self.screenSize.height - 64), collectionViewLayout:flowLayot);
        self.assetView!.backgroundView = nil;
        self.assetView!.backgroundColor = UIColor.whiteColor();
        self.assetView!.delegate = self;
        self.assetView!.dataSource = self;
        self.assetView!.showsVerticalScrollIndicator = false;
        self.assetView!.showsHorizontalScrollIndicator = false;
        self.assetView!.registerClass(LBAssetCell.self, forCellWithReuseIdentifier:self.cellId);
        self.assetView!.registerClass(LBAssetTitleView.self, forSupplementaryViewOfKind:UICollectionElementKindSectionFooter, withReuseIdentifier:self.headId);
        self.view.addSubview(self.assetView!);
    }
    
    override func viewWillAppear(animated: Bool) {
        super.viewWillAppear(animated);
        if !self.isUnFirst {
            self.isUnFirst = true;
            initData();
        }
    }

    override func didReceiveMemoryWarning() {
        super.didReceiveMemoryWarning()
        // Dispose of any resources that can be recreated.
    }
    
    func finishPickingAssets() {
        self.navigationItem.rightBarButtonItem?.enabled = false;
        let picker : LBAssetPickerController = self.navigationController as! LBAssetPickerController;
        
        let chooseList : NSMutableArray = NSMutableArray(capacity: 2);
        for i : NSInteger in 0..<self.selectedItemList!.count {
            let asset : ALAsset = self.assetList!.objectAtIndex(i) as! ALAsset;
            if (self.selectedItemList!.objectAtIndex(i) as! String) == "1" {
                chooseList.addObject(asset);
            }
        }
        if chooseList.count < picker.minNumOfSelection {
            picker.pickerDelegate?.assetPickerControllerDiMinimun?(picker);
        }
        
        if chooseList.count > picker.maxNumOfSelection {
            picker.pickerDelegate?.assetPickerControllerDiMaximun?(picker);
        }
        
        picker.pickerDelegate?.assetPickerController(picker, didFinishedPickedAssets:chooseList);
        
        weak var weakSelf : LBAssetViewController? = self;
        dispatch_after(dispatch_time(DISPATCH_TIME_NOW,Int64(Double(NSEC_PER_SEC) * 0.2)),
                       dispatch_get_main_queue(), {
                        if picker.isFinishDismissViewController {
                            weakSelf?.navigationItem.rightBarButtonItem?.enabled = true;
                            picker.dismissViewControllerAnimated(true, completion:nil);
                        }
        });
    }
    
    //MARK:- 构建数据
    private func initData() {
        if self.assetGroup == nil {
            return;
        }
        
        self.photoNum = 0;
        self.videoNum = 0;
        self.choosePhotoNum = 0;
        self.chooseVideoNum = 0;
        self.selectedItemList?.removeAllObjects();
        self.assetList?.removeAllObjects();
        self.assetTitle = self.assetGroup!.valueForProperty(ALAssetsGroupPropertyName) as? String
        self.title = self.assetTitle;
        
        let resultsBlock : ALAssetsGroupEnumerationResultsBlock = { (asset : ALAsset!, index : Int, stop : UnsafeMutablePointer<ObjCBool>) in
            if asset != nil {
                self.assetList!.addObject(asset);
                self.selectedItemList!.addObject("0");
                
                let aType : String = asset.valueForProperty(ALAssetPropertyType) as! String;
                if aType == ALAssetTypePhoto {
                    self.photoNum += 1;
                }else if aType == ALAssetTypeVideo {
                    self.videoNum += 1;
                }
            }else if (self.assetList!.count > 0) {
                self.assetView!.reloadData();
                self.assetView!.scrollToItemAtIndexPath(NSIndexPath(forRow:self.assetList!.count - 1, inSection:0),
                                                        atScrollPosition:.Top,
                                                        animated:true);
            }
        };
        self.assetGroup!.enumerateAssetsUsingBlock(resultsBlock);
    }
    
    //MARK:- UICollectionView
    func numberOfSectionsInCollectionView(collectionView: UICollectionView) -> Int {
        return 1;
    }
    
    func collectionView(collectionView: UICollectionView, numberOfItemsInSection section: Int) -> Int {
        return self.assetList!.count;
    }
    
    func collectionView(collectionView: UICollectionView, layout collectionViewLayout: UICollectionViewLayout, sizeForItemAtIndexPath indexPath: NSIndexPath) -> CGSize {
        return CGSizeMake(self.itemWidth,self.itemWidth);
    }
    
    func collectionView(collectionView: UICollectionView, layout collectionViewLayout: UICollectionViewLayout, referenceSizeForFooterInSection section: Int) -> CGSize {
        return CGSizeMake(collectionView.frame.size.width,36);
    }
    
    func collectionView(collectionView: UICollectionView, layout collectionViewLayout: UICollectionViewLayout, insetForSectionAtIndex section: Int) -> UIEdgeInsets {
        return UIEdgeInsetsMake(self.leftSpacing,self.leftSpacing,0,self.leftSpacing);
    }
    
    func collectionView(collectionView: UICollectionView, layout collectionViewLayout: UICollectionViewLayout, minimumLineSpacingForSectionAtIndex section: Int) -> CGFloat {
        return self.miniLineSpacing;
    }
    
    func collectionView(collectionView: UICollectionView, layout collectionViewLayout: UICollectionViewLayout, minimumInteritemSpacingForSectionAtIndex section: Int) -> CGFloat {
        return self.miniItemSpacing;
    }
    
    func collectionView(collectionView: UICollectionView, cellForItemAtIndexPath indexPath: NSIndexPath) -> UICollectionViewCell {
        let cell : LBAssetCell = collectionView.dequeueReusableCellWithReuseIdentifier(self.cellId, forIndexPath: indexPath) as! LBAssetCell;
        let picker : LBAssetPickerController = self.navigationController as! LBAssetPickerController;
        if indexPath.row < self.assetList!.count {
            let asset : ALAsset = self.assetList!.objectAtIndex(indexPath.row) as! ALAsset
            cell.bindData(asset);
            cell.isDisabled = !picker.selectionFilter.evaluateWithObject(asset);
            cell.isChoosed = ((self.selectedItemList!.objectAtIndex(indexPath.row) as! String) == "1");
        }
        
        return cell;
    }
    
    func collectionView(collectionView: UICollectionView, viewForSupplementaryElementOfKind kind: String, atIndexPath indexPath: NSIndexPath) -> UICollectionReusableView {
        let aView : UICollectionReusableView = collectionView.dequeueReusableSupplementaryViewOfKind(kind, withReuseIdentifier:self.headId, forIndexPath:indexPath);
        if (kind == UICollectionElementKindSectionFooter && aView.isKindOfClass(LBAssetTitleView.self)) {
            let aStr : String = String(format:"%d张照片,%d部视频",self.photoNum,self.videoNum);
            (aView as! LBAssetTitleView).setTitle(aStr);
        }
        return aView;
    }
    
    func collectionView(collectionView: UICollectionView, didSelectItemAtIndexPath indexPath: NSIndexPath) {
        if indexPath.row < self.selectedItemList!.count {
            let picker : LBAssetPickerController = self.navigationController as! LBAssetPickerController;
            let cell : LBAssetCell = collectionView.dequeueReusableCellWithReuseIdentifier(self.cellId, forIndexPath: indexPath) as! LBAssetCell;
            if cell.isDisabled {
                return;
            }
            
            var chooseState : String = self.selectedItemList!.objectAtIndex(indexPath.row) as! String;
            let asset : ALAsset = self.assetList!.objectAtIndex(indexPath.row) as! ALAsset;
            let aType : String = asset.valueForProperty(ALAssetPropertyType) as! String;
            var addNum : NSInteger = 0;
            
            if chooseState == "0" {
                chooseState = "1";
                addNum = 1;
            }else{
                chooseState = "0";
                addNum = -1;
            }
            
            if self.choosePhotoNum + self.chooseVideoNum + addNum > picker.maxNumOfSelection {
                let alert : UIAlertView = UIAlertView(title:"提示", message:"已选满，不能再选", delegate:nil, cancelButtonTitle:"确定");
                alert.show();
                return;
            }
            
            if aType == ALAssetTypePhoto {
                self.choosePhotoNum += addNum;
            }else{
                self.chooseVideoNum += addNum;
            }
            self.selectedItemList!.replaceObjectAtIndex(indexPath.row, withObject:chooseState);
            self.assetView!.reloadData();
            
            var aStr : String? = "";
            if self.chooseVideoNum > 0 && self.choosePhotoNum > 0 {
                aStr = String(format:"选择了%d张照片,%d部视频",self.choosePhotoNum,self.chooseVideoNum);
            }else if self.choosePhotoNum > 0 {
                aStr = String(format:"选择了%d张照片",self.choosePhotoNum);
            }else if self.chooseVideoNum > 0 {
                aStr = String(format:"选择了%d部视频",self.chooseVideoNum);
            }else{
                aStr = self.assetTitle;
            }
            self.title = aStr;
        }
    }

}
