//
//  LBImageChooser.swift
//  LBAssetPickerDemo
//
//  Created by zhubaotown on 16/4/21.
//  Copyright © 2016年 zhubaotown. All rights reserved.
//

import UIKit
import AssetsLibrary

@objc protocol LBImageChooserDelegate {
    /*
     选择图片后将会保存在沙盒中，再保存之前需要获取保存的路径,该方法获取保存的文件名
     */
    func imageChooserWillSave() -> String;
    
    func imageChooserFinished();
    
    optional func imageChooserDidSave(path : String,withImageInfo imgInfo : NSDictionary?);
    
    optional func imageChooserScaleImage(aImage : UIImage) -> UIImage?;
}

class LBImageChooser:NSObject,
    UIActionSheetDelegate,
UIImagePickerControllerDelegate,
UINavigationControllerDelegate,
LBAssetPickerViewControllerDelegate {
    
    private var imageCount : NSInteger = 10;
    private weak var delegate : LBImageChooserDelegate? = nil;
    private weak var rootController : UIViewController? = nil;
    private var waitView : UIActivityIndicatorView? = nil;
    
    deinit {
        self.delegate = nil;
    }

    override init() {
        super.init();
        self.imageCount = 10;
    }
    
    //选择图片
    func chooseImageInViewController(aController : UIViewController!,totalCount maxNum : NSInteger,withDelegate aDelegate : LBImageChooserDelegate?) {
        self.rootController = aController;
        self.delegate = aDelegate;
        self.imageCount = maxNum;
        
        let actionSheet : UIActionSheet = UIActionSheet(title:"请选择照片",
                                                        delegate:self,
                                                        cancelButtonTitle:"取消",
                                                        destructiveButtonTitle:"使用相机拍照",
                                                        otherButtonTitles:"使用相册中的照片");
        actionSheet.actionSheetStyle = .BlackOpaque;
        actionSheet.destructiveButtonIndex = -1;
        actionSheet.showInView((UIApplication.sharedApplication().delegate!.window!)!);
    }
    
    //拍照选择图片
    func chooseImageFromCameraInViewController(aController : UIViewController!,totalCount maxNum : NSInteger,withDelegate aDelegate : LBImageChooserDelegate?) {
        if nil == self.rootController {
            self.rootController = aController;
        }
        self.delegate = aDelegate;
        self.imageCount = maxNum;
        
        startChoose();
        
        let pickController : UIImagePickerController = UIImagePickerController();
        pickController.sourceType = .Camera;
        pickController.allowsEditing = false;
        pickController.delegate = self;
        self.rootController?.presentViewController(pickController, animated:true, completion:nil);
    }
    
    //从相册中选择图片
    func chooseImageFromAlbumInViewController(aController : UIViewController!,totalCount maxNum : NSInteger,withDelegate aDelegate : LBImageChooserDelegate?) {
        if nil == self.rootController {
            self.rootController = aController;
        }
        self.delegate = aDelegate;
        self.imageCount = maxNum;
        
        startChoose();
        
        let pickController : LBAssetPickerController = LBAssetPickerController.defaultPickerController();
        pickController.maxNumOfSelection = maxNum;
        pickController.assetsFilter = ALAssetsFilter.allPhotos();
        pickController.pickerDelegate = self;
        pickController.selectionFilter = NSPredicate(block: { (evaluatedObject : AnyObject, bindings : [String : AnyObject]?) -> Bool in
            if (evaluatedObject.isKindOfClass(ALAsset.self)) {
                let aObj : ALAsset = (evaluatedObject as! ALAsset);
                if aObj.valueForProperty(ALAssetPropertyType).isEqual(ALAssetTypeVideo) {
                    let duration : NSTimeInterval = aObj.valueForProperty(ALAssetPropertyDuration).doubleValue;
                    return duration >= 5.0;
                }else{
                    return true;
                }
            }else{
                return true;
            }
        });
        self.rootController?.presentViewController(pickController, animated:true, completion:nil);
    }
    
    //MARK:- ActionSheet
    func actionSheet(actionSheet: UIActionSheet, didDismissWithButtonIndex buttonIndex: Int) {
        if buttonIndex == 1 {
            chooseImageFromCameraInViewController(self.rootController,
                                                  totalCount:self.imageCount,
                                                  withDelegate:self.delegate);
        }else if buttonIndex == 2 {
            chooseImageFromAlbumInViewController(self.rootController,
                                                 totalCount:self.imageCount,
                                                 withDelegate:self.delegate);
        }
    }
    
    //MARK: chooseResult
    private func saveImageWithName(aName : String?,withImageData imgData : NSData?) -> String {
        if aName == nil || (aName! as NSString).length == 0 {
            return "";
        }
        
        if imgData == nil || imgData?.length == 0 {
            return "";
        }
        
        let basePath : String = (NSTemporaryDirectory() as NSString).stringByAppendingPathComponent("imageTemp");
        var ret : Bool = NSFileManager.defaultManager().fileExistsAtPath(basePath);
        if !ret {
            do {
                try NSFileManager.defaultManager().createDirectoryAtPath(basePath,
                                                                         withIntermediateDirectories:false,
                                                                         attributes:nil);
                ret = true;
            }catch{
                ret = false;
            }
        }
        
        if !ret {
            return "";
        }
        
        let filePath : String = (basePath as NSString).stringByAppendingPathComponent(aName!);
        do {
            try imgData!.writeToFile(filePath, options: NSDataWritingOptions.AtomicWrite);
            ret = true;
        }catch {
            ret = false;
        }
        
        if ret {
            return filePath;
        }
        return "";
    }
    
    func chooseImageWithData(aData : NSData?,withImageInfo imgInfo : NSDictionary?) {
        if self.delegate != nil {
            let imgName : String = self.delegate!.imageChooserWillSave();
            let savePath : String = saveImageWithName(imgName, withImageData:aData);
            if (savePath as NSString).length != 0 {
                self.delegate!.imageChooserDidSave?(savePath, withImageInfo:imgInfo);
            }
        }
    }
    
    private func startChoose() {
        if self.rootController == nil {
            return;
        }
        self.waitView = UIActivityIndicatorView(frame:CGRectMake(0,0,40,40));
        self.waitView?.center = (self.rootController?.view.center)!;
        self.waitView?.activityIndicatorViewStyle = .Gray;
        self.waitView?.hidesWhenStopped = true;
        self.rootController!.view.addSubview(self.waitView!);
    }
    
    func finishedChoose() {
        self.waitView?.stopAnimating();
        self.waitView?.removeFromSuperview();
        self.waitView = nil;
        self.delegate?.imageChooserFinished();
    }
    
    //MARK:- UIImagePickerController 
    func imagePickerController(picker: UIImagePickerController, didFinishPickingMediaWithInfo info: [String : AnyObject]) {
        let mediaType : String? = info[UIImagePickerControllerMediaType] as? String;
        weak var imgInfo : NSDictionary? = info[UIImagePickerControllerMediaMetadata] as? NSDictionary;
        weak var pImg : UIImage? = nil;
        if mediaType != nil && mediaType! == "public.image" {
            pImg = info[UIImagePickerControllerOriginalImage] as? UIImage;
        }
        
        weak var weakSelf : LBImageChooser? = self;
        picker.dismissViewControllerAnimated(true) { 
            weakSelf?.dealChooseImage(pImg, withImageInfo:imgInfo);
        }
    }
    
    func dealChooseImage(aImg : UIImage?,withImageInfo imgInfo : NSDictionary?) {
        weak var weakSelf : LBImageChooser? = self;
        if aImg == nil {
            weakSelf?.finishedChoose();
        }else{
            dispatch_async(dispatch_get_global_queue(DISPATCH_QUEUE_PRIORITY_DEFAULT,0), {
                var resultImg : UIImage = aImg!.fixOrientation();
                if weakSelf?.delegate != nil {
                    let temImg : UIImage? = weakSelf?.delegate!.imageChooserScaleImage?(resultImg);
                    if temImg != nil {
                        resultImg = temImg!;
                    }
                }
                weakSelf?.chooseImageWithData(UIImageJPEGRepresentation(resultImg,0.8), withImageInfo:imgInfo);
                dispatch_async(dispatch_get_main_queue(), {
                    weakSelf?.finishedChoose();
                });
            });
        }
    }
    
    //MARK:- LBAssets
    func assetPickerController(picker: LBAssetPickerController, didFinishedPickedAssets assets: NSArray?) {
        weak var weakSelf : LBImageChooser? = self;
        dispatch_async(dispatch_get_global_queue(DISPATCH_QUEUE_PRIORITY_DEFAULT,0), {
            if (assets != nil && assets!.count > 0) {
                for i : NSInteger in 0..<assets!.count {
                    autoreleasepool({ 
                        let assetObj : ALAsset = assets![i] as! ALAsset;
                        var resultImg : UIImage = UIImage(CGImage:assetObj.defaultRepresentation().fullScreenImage().takeUnretainedValue());
                        if weakSelf?.delegate != nil {
                            let temImg : UIImage? = weakSelf?.delegate!.imageChooserScaleImage?(resultImg);
                            if temImg != nil {
                                resultImg = temImg!;
                            }
                        }
                        
                        weakSelf?.chooseImageWithData(UIImageJPEGRepresentation(resultImg,0.8), withImageInfo:assetObj.defaultRepresentation().metadata());
                    });
                }
            }
            
            dispatch_async(dispatch_get_main_queue(), {
                weakSelf?.finishedChoose();
            });
        });
    }
}

extension UIImage {
    func fixOrientation() -> UIImage {
        // No-op if the orientation is already correct
        if (self.imageOrientation == UIImageOrientation.Up) {
            return self;
        }
        
        // We need to calculate the proper transformation to make the image upright.
        // We do it in 2 steps: Rotate if Left/Right/Down, and then flip if Mirrored.
        var transform : CGAffineTransform  = CGAffineTransformIdentity;
        
        switch (self.imageOrientation){
        case .Down,.DownMirrored:
            transform = CGAffineTransformTranslate(transform, self.size.width, self.size.height);
            transform = CGAffineTransformRotate(transform, CGFloat(M_PI));
            break;
            
        case .Left,.LeftMirrored:
            transform = CGAffineTransformTranslate(transform, self.size.width, 0);
            transform = CGAffineTransformRotate(transform, CGFloat(M_PI_2));
            break;
            
        case .Right,.RightMirrored:
            transform = CGAffineTransformTranslate(transform, 0, self.size.height);
            transform = CGAffineTransformRotate(transform, -CGFloat(M_PI_2));
            break;
        default:
            break;
        }
        
        switch (self.imageOrientation) {
        case .UpMirrored,.DownMirrored:
            transform = CGAffineTransformTranslate(transform, self.size.width, 0);
            transform = CGAffineTransformScale(transform, -1, 1);
            break;
            
        case .LeftMirrored,.RightMirrored:
            transform = CGAffineTransformTranslate(transform, self.size.height, 0);
            transform = CGAffineTransformScale(transform, -1, 1);
            break;
        default:
            break;
        }
        
        // Now we draw the underlying CGImage into a new context, applying the transform
        // calculated above.
        let ctx : CGContextRef?  = CGBitmapContextCreate(nil,
                                                        Int(self.size.width),Int(self.size.height),
                                                        CGImageGetBitsPerComponent(self.CGImage), 0,
                                                        CGImageGetColorSpace(self.CGImage),
                                                        CGImageGetBitmapInfo(self.CGImage).rawValue);
        CGContextConcatCTM(ctx, transform);
        switch (self.imageOrientation) {
        case .Left,.LeftMirrored,.Right,.RightMirrored:
            // Grr...
            CGContextDrawImage(ctx, CGRectMake(0,0,self.size.height,self.size.width), self.CGImage);
            break;
            
        default:
            CGContextDrawImage(ctx, CGRectMake(0,0,self.size.width,self.size.height), self.CGImage);
            break;
        }
        
        // And now we just create a new UIImage from the drawing context
        let cgimg : CGImageRef?  = CGBitmapContextCreateImage(ctx);
        let img : UIImage = UIImage(CGImage: cgimg!);
        return img;
    }
}
